@@ -52,7 +52,7 @@ DEBUG_GET_ONCE_BOOL_OPTION(draw_no_fse, "DRAW_NO_FSE", FALSE)
  *     - backend  -- the vbuf_render provided by the driver.
  */
 static boolean
-draw_pt_arrays(struct draw_context *draw, 
+draw_pt_arrays(struct draw_context *draw,
                unsigned prim,
                unsigned start, 
                unsigned count)
@@ -106,17 +106,68 @@ draw_pt_arrays(struct draw_context *draw,
          middle = draw->pt.middle.general;
    }
 
-   frontend = draw->pt.front.vsplit;
+   frontend = draw->pt.frontend;
 
-   frontend->prepare( frontend, prim, middle, opt );
+   if (frontend ) {
+      if (draw->pt.prim != prim || draw->pt.opt != opt) {
+         /* In certain conditions switching primitives requires us to flush
+          * and validate the different stages. One example is when smooth
+          * lines are active but first drawn with triangles and then with
+          * lines.
+          */
+         draw_do_flush( draw, DRAW_FLUSH_STATE_CHANGE );
+         frontend = NULL;
+      } else if (draw->pt.eltSize != draw->pt.user.eltSize) {
+         /* Flush draw state if eltSize changed.
+          * This could be improved so only the frontend is flushed since it
+          * converts all indices to ushorts and the fetch part of the middle
+          * always prepares both linear and indexed.
+          */
+         frontend->flush( frontend, DRAW_FLUSH_STATE_CHANGE );
+         frontend = NULL;
+      }
+   }
 
-   frontend->run(frontend, start, count);
+   if (!frontend) {
+      frontend = draw->pt.front.vsplit;
 
-   frontend->finish( frontend );
+      frontend->prepare( frontend, prim, middle, opt );
+
+      draw->pt.frontend = frontend;
+      draw->pt.eltSize = draw->pt.user.eltSize;
+      draw->pt.prim = prim;
+      draw->pt.opt = opt;
+   }
+
+   if (draw->pt.rebind_parameters) {
+      /* update constants, viewport dims, clip planes, etc */
+      middle->bind_parameters(middle);
+      draw->pt.rebind_parameters = FALSE;
+   }
+
+   frontend->run( frontend, start, count );
 
    return TRUE;
 }
 
+void draw_pt_flush( struct draw_context *draw, unsigned flags )
+{
+   assert(flags);
+
+   if (draw->pt.frontend) {
+      draw->pt.frontend->flush( draw->pt.frontend, flags );
+
+      /* don't prepare if we only are flushing the backend */
+      if (flags & DRAW_FLUSH_STATE_CHANGE)
+         draw->pt.frontend = NULL;
+   }
+
+   if (flags & DRAW_FLUSH_PARAMETER_CHANGE) {
+      draw->pt.rebind_parameters = TRUE;
+   }
+}
+
+
 
 boolean draw_pt_init( struct draw_context *draw )
 {
@@ -193,28 +244,24 @@ draw_print_arrays(struct draw_context *draw, uint prim, int start, uint count)
       uint j;
 
       if (draw->pt.user.eltSize) {
-         const char *elts;
-
          /* indexed arrays */
-         elts = (const char *) draw->pt.user.elts;
-         elts += draw->pt.index_buffer.offset;
 
          switch (draw->pt.user.eltSize) {
          case 1:
             {
-               const ubyte *elem = (const ubyte *) elts;
+               const ubyte *elem = (const ubyte *) draw->pt.user.elts;
                ii = elem[start + i];
             }
             break;
          case 2:
             {
-               const ushort *elem = (const ushort *) elts;
+               const ushort *elem = (const ushort *) draw->pt.user.elts;
                ii = elem[start + i];
             }
             break;
          case 4:
             {
-               const uint *elem = (const uint *) elts;
+               const uint *elem = (const uint *) draw->pt.user.elts;
                ii = elem[start + i];
             }
             break;
@@ -327,30 +374,27 @@ draw_pt_arrays_restart(struct draw_context *draw,
 
    assert(info->primitive_restart);
 
-   if (draw->pt.user.elts) {
+   if (draw->pt.user.eltSize) {
       /* indexed prims (draw_elements) */
-      const char *elts =
-         (const char *) draw->pt.user.elts + draw->pt.index_buffer.offset;
-
       cur_start = start;
       cur_count = 0;
 
       switch (draw->pt.user.eltSize) {
       case 1:
          {
-            const ubyte *elt_ub = (const ubyte *) elts;
+            const ubyte *elt_ub = (const ubyte *) draw->pt.user.elts;
             PRIM_RESTART_LOOP(elt_ub);
          }
          break;
       case 2:
          {
-            const ushort *elt_us = (const ushort *) elts;
+            const ushort *elt_us = (const ushort *) draw->pt.user.elts;
             PRIM_RESTART_LOOP(elt_us);
          }
          break;
       case 4:
          {
-            const uint *elt_ui = (const uint *) elts;
+            const uint *elt_ui = (const uint *) draw->pt.user.elts;
             PRIM_RESTART_LOOP(elt_ui);
          }
          break;
@@ -401,12 +445,8 @@ draw_arrays_instanced(struct draw_context *draw,
    info.count = count;
    info.start_instance = startInstance;
    info.instance_count = instanceCount;
-
-   info.indexed = (draw->pt.user.elts != NULL);
-   if (!info.indexed) {
-      info.min_index = start;
-      info.max_index = start + count - 1;
-   }
+   info.min_index = start;
+   info.max_index = start + count - 1;
 
    draw_vbo(draw, &info);
 }
@@ -415,33 +455,24 @@ draw_arrays_instanced(struct draw_context *draw,
 /**
  * Draw vertex arrays.
  * This is the main entrypoint into the drawing module.  If drawing an indexed
- * primitive, the draw_set_index_buffer() and draw_set_mapped_index_buffer()
- * functions should have already been called to specify the element/index
- * buffer information.
+ * primitive, the draw_set_indexes() function should have already been called
+ * to specify the element/index buffer information.
  */
 void
 draw_vbo(struct draw_context *draw,
          const struct pipe_draw_info *info)
 {
-   unsigned reduced_prim = u_reduced_prim(info->mode);
    unsigned instance;
    unsigned index_limit;
-
+   unsigned count;
    assert(info->instance_count > 0);
    if (info->indexed)
       assert(draw->pt.user.elts);
 
-   draw->pt.user.eltSize =
-      (info->indexed) ? draw->pt.index_buffer.index_size : 0;
-
    draw->pt.user.eltBias = info->index_bias;
    draw->pt.user.min_index = info->min_index;
    draw->pt.user.max_index = info->max_index;
-
-   if (reduced_prim != draw->reduced_prim) {
-      draw_do_flush(draw, DRAW_FLUSH_STATE_CHANGE);
-      draw->reduced_prim = reduced_prim;
-   }
+   draw->pt.user.eltSize = info->indexed ? draw->pt.user.eltSizeIB : 0;
 
    if (0)
       debug_printf("draw_vbo(mode=%u start=%u count=%u):\n",
@@ -475,7 +506,6 @@ draw_vbo(struct draw_context *draw,
       draw_print_arrays(draw, info->mode, info->start, MIN2(info->count, 20));
 
    index_limit = util_draw_max_index(draw->pt.vertex_buffer,
-                                     draw->pt.nr_vertex_buffers,
                                      draw->pt.vertex_element,
                                      draw->pt.nr_vertex_elements,
                                      info);
@@ -488,6 +518,11 @@ draw_vbo(struct draw_context *draw,
 
    draw->pt.max_index = index_limit - 1;
 
+   count = info->count;
+   if (count == 0) {
+      if (info->count_from_stream_output)
+         count = draw->pt.max_index + 1;
+   }
 
    /*
     * TODO: We could use draw->pt.max_index to further narrow
@@ -501,7 +536,7 @@ draw_vbo(struct draw_context *draw,
          draw_pt_arrays_restart(draw, info);
       }
       else {
-         draw_pt_arrays(draw, info->mode, info->start, info->count);
+         draw_pt_arrays(draw, info->mode, info->start, count);
       }
    }
 }
