@@ -35,9 +35,16 @@
 #ifndef _UTIL_CPU_DETECT_H
 #define _UTIL_CPU_DETECT_H
 
+
 #include "pipe/p_compiler.h"
 #include "pipe/p_config.h"
 
+
+#ifdef	__cplusplus
+extern "C" {
+#endif
+
+
 struct util_cpu_caps {
    unsigned nr_cpus;
 
@@ -45,6 +52,7 @@ struct util_cpu_caps {
    int x86_cpu_type;
    unsigned cacheline;
 
+   unsigned has_intel:1;
    unsigned has_tsc:1;
    unsigned has_mmx:1;
    unsigned has_mmx2:1;
@@ -66,4 +74,9 @@ util_cpu_caps;
 void util_cpu_detect(void);
 
 
+#ifdef	__cplusplus
+}
+#endif
+
+
 #endif /* _UTIL_CPU_DETECT_H */
