@@ -52,9 +52,10 @@ VERY_LARGE = 99999999999999999999999
 class Channel:
     '''Describe the channel of a color channel.'''
     
-    def __init__(self, type, norm, size, name = ''):
+    def __init__(self, type, norm, pure, size, name = ''):
         self.type = type
         self.norm = norm
+        self.pure = pure
         self.size = size
         self.sign = type in (SIGNED, FIXED, FLOAT)
         self.name = name
@@ -63,11 +64,13 @@ class Channel:
         s = str(self.type)
         if self.norm:
             s += 'n'
+        if self.pure:
+            s += 'p'
         s += str(self.size)
         return s
 
     def __eq__(self, other):
-        return self.type == other.type and self.norm == other.norm and self.size == other.size
+        return self.type == other.type and self.norm == other.norm and self.pure == other.pure and self.size == other.size
 
     def max(self):
         '''Maximum representable number.'''
@@ -141,9 +144,18 @@ class Format:
         if self.layout != PLAIN:
             return False
         ref_channel = self.channels[0]
-        for channel in self.channels[1:]:
+        if ref_channel.type == VOID:
+           ref_channel = self.channels[1]
+        for channel in self.channels:
             if channel.size and (channel.size != ref_channel.size or channel.size % 8):
                 return False
+            if channel.type != VOID:
+                if channel.type != ref_channel.type:
+                    return False
+                if channel.norm != ref_channel.norm:
+                    return False
+                if channel.pure != ref_channel.pure:
+                    return False
         return True
 
     def is_mixed(self):
@@ -158,6 +170,8 @@ class Format:
                     return True
                 if channel.norm != ref_channel.norm:
                     return True
+                if channel.pure != ref_channel.pure:
+                    return True
         return False
 
     def is_pot(self):
@@ -191,10 +205,11 @@ class Format:
 
     def inv_swizzles(self):
         '''Return an array[4] of inverse swizzle terms'''
+        '''Only pick the first matching value to avoid l8 getting blue and i8 getting alpha'''
         inv_swizzle = [None]*4
         for i in range(4):
             swizzle = self.swizzles[i]
-            if swizzle < 4:
+            if swizzle < 4 and inv_swizzle[swizzle] == None:
                 inv_swizzle[swizzle] = i
         return inv_swizzle
 
@@ -274,15 +289,22 @@ def parse(filename):
                 type = _type_parse_map[field[0]]
                 if field[1] == 'n':
                     norm = True
+                    pure = False
+                    size = int(field[2:])
+                elif field[1] == 'p':
+                    pure = True
+                    norm = False
                     size = int(field[2:])
                 else:
                     norm = False
+                    pure = False
                     size = int(field[1:])
             else:
                 type = VOID
                 norm = False
+                pure = False
                 size = 0
-            channel = Channel(type, norm, size, names[i])
+            channel = Channel(type, norm, pure, size, names[i])
             channels.append(channel)
 
         format = Format(name, layout, block_width, block_height, channels, swizzles, colorspace)
