@@ -36,7 +36,6 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "main/glheader.h"
 
 #include "radeon_context.h"
-#include "radeon_ioctl.h"
 #include "radeon_sanity.h"
 
 /* Set this '1' to get more verbiage.
@@ -44,11 +43,11 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define MORE_VERBOSE 1
 
 #if MORE_VERBOSE
-#define VERBOSE (RADEON_DEBUG & DEBUG_VERBOSE)
+#define VERBOSE (RADEON_DEBUG & RADEON_VERBOSE)
 #define NORMAL  (1)
 #else
 #define VERBOSE 0
-#define NORMAL  (RADEON_DEBUG & DEBUG_VERBOSE)
+#define NORMAL  (RADEON_DEBUG & RADEON_VERBOSE)
 #endif
 
 
@@ -381,8 +380,7 @@ static int find_or_add_value( struct reg *reg, int val )
    if (j == reg->nalloc) {
       reg->nalloc += 5;
       reg->nalloc *= 2;
-      reg->values = (union fi *) realloc( reg->values, 
-					  reg->nalloc * sizeof(union fi) );
+      reg->values = realloc( reg->values, reg->nalloc * sizeof(union fi) );
    }
 
    reg->values[reg->nvalues++].i = val;
@@ -973,7 +971,7 @@ static int radeon_emit_packet3_cliprect( drm_radeon_cmd_buffer_t *cmdbuf )
 }
 
 
-int radeonSanityCmdBuffer( radeonContextPtr rmesa,
+int radeonSanityCmdBuffer( r100ContextPtr rmesa,
 			   int nbox,
 			   drm_clip_rect_t *boxes )
 {
