@@ -28,10 +28,15 @@
 
 
 struct _glapi_table;
+struct gl_context;
 
+extern struct _glapi_table *
+_mesa_alloc_dispatch_table(void);
 
 extern void
-_mesa_init_exec_table(struct _glapi_table *exec);
+_mesa_initialize_exec_table(struct gl_context *ctx);
 
+extern void
+_mesa_initialize_dispatch_tables(struct gl_context *ctx);
 
 #endif
