@@ -29,6 +29,8 @@
 #define ARRAYOBJ_H
 
 #include "glheader.h"
+#include "mtypes.h"
+#include "glformats.h"
 
 struct gl_context;
 
@@ -51,9 +53,19 @@ extern void
 _mesa_delete_array_object( struct gl_context *ctx, struct gl_array_object *obj );
 
 extern void
+_mesa_reference_array_object_(struct gl_context *ctx,
+                              struct gl_array_object **ptr,
+                              struct gl_array_object *arrayObj);
+
+static inline void
 _mesa_reference_array_object(struct gl_context *ctx,
                              struct gl_array_object **ptr,
-                             struct gl_array_object *arrayObj);
+                             struct gl_array_object *arrayObj)
+{
+   if (*ptr != arrayObj)
+      _mesa_reference_array_object_(ctx, ptr, arrayObj);
+}
+
 
 extern void
 _mesa_initialize_array_object( struct gl_context *ctx,
@@ -64,6 +76,30 @@ extern void
 _mesa_update_array_object_max_element(struct gl_context *ctx,
                                       struct gl_array_object *arrayObj);
 
+/** Returns the bitmask of all enabled arrays in fixed function mode.
+ *
+ *  In fixed function mode only the traditional fixed function arrays
+ *  are available.
+ */
+static inline GLbitfield64
+_mesa_array_object_get_enabled_ff(const struct gl_array_object *arrayObj)
+{
+   return arrayObj->_Enabled & VERT_BIT_FF_ALL;
+}
+
+/** Returns the bitmask of all enabled arrays in arb/glsl shader mode.
+ *
+ *  In arb/glsl shader mode all the fixed function and the arb/glsl generic
+ *  arrays are available. Only the first generic array takes
+ *  precedence over the legacy position array.
+ */
+static inline GLbitfield64
+_mesa_array_object_get_enabled_arb(const struct gl_array_object *arrayObj)
+{
+   GLbitfield64 enabled = arrayObj->_Enabled;
+   return enabled & ~(VERT_BIT_POS & (enabled >> VERT_ATTRIB_GENERIC0));
+}
+
 
 /*
  * API functions
@@ -74,12 +110,12 @@ void GLAPIENTRY _mesa_BindVertexArray( GLuint id );
 
 void GLAPIENTRY _mesa_BindVertexArrayAPPLE( GLuint id );
 
-void GLAPIENTRY _mesa_DeleteVertexArraysAPPLE(GLsizei n, const GLuint *ids);
+void GLAPIENTRY _mesa_DeleteVertexArrays(GLsizei n, const GLuint *ids);
 
 void GLAPIENTRY _mesa_GenVertexArrays(GLsizei n, GLuint *arrays);
 
 void GLAPIENTRY _mesa_GenVertexArraysAPPLE(GLsizei n, GLuint *buffer);
 
-GLboolean GLAPIENTRY _mesa_IsVertexArrayAPPLE( GLuint id );
+GLboolean GLAPIENTRY _mesa_IsVertexArray( GLuint id );
 
 #endif /* ARRAYOBJ_H */
