@@ -29,10 +29,7 @@
 
 #include "swrast/s_context.h"
 
-extern FetchTexelFunc
-_mesa_get_texel_fetch_func(gl_format format, GLuint dims);
-
 void
-_mesa_update_fetch_functions(struct gl_texture_object *texObj);
+_mesa_update_fetch_functions(struct gl_context *ctx, GLuint unit);
 
 #endif /* S_TEXFETCH_H */
